@@ -1,8 +1,8 @@
-// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; tab-width: 8 -*-
+// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 //
 // Formula.h: Rcpp R/C++ interface class library -- formula
 //
-// Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2012 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -27,19 +27,22 @@
 
 namespace Rcpp{ 
 
-class Formula : public Language {
-public:
+    class Formula : public Language {
+    public:
 
-	Formula() ;
-	
-	Formula(SEXP lang) throw(not_compatible) ;
+        Formula() ;
+        
+        Formula(SEXP lang);
 
-	Formula(const Formula& other) ;
-	explicit Formula( const std::string& code ) throw(not_compatible) ; 
+        Formula(const Formula& other) ;
+        explicit Formula( const std::string& code ); 
 
-	Formula& operator=(const Formula& other) ;
-	
-} ; 
+        Formula& operator=(const Formula& other) ;
+        
+    private:
+        void set_sexp(SEXP x) ;
+        
+    } ; 
 
 } // namespace Rcpp
 
