@@ -2,7 +2,7 @@
 //
 // pmax.h: Rcpp R/C++ interface class library -- pmax
 //
-// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2012 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -25,66 +25,48 @@
 namespace Rcpp{
 namespace sugar{
 
-template <int RTYPE, bool LHS_NA, bool RHS_NA> class pmax_op {
-public:
-	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
-	
-	inline STORAGE operator()( STORAGE left, STORAGE right ) const {
-		if( Rcpp::traits::is_na<RTYPE>(left) ) return left ;
-		if( Rcpp::traits::is_na<RTYPE>(right) ) return right ;
-		return left > right ? left : right ;
-	}
-	
-} ;
-template <int RTYPE, bool LHS_NA> class pmax_op<RTYPE,LHS_NA,false> {
-public:
-	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
-	
-	inline STORAGE operator()( STORAGE left, STORAGE right ) const {
-		if( Rcpp::traits::is_na<RTYPE>(left) ) return left ;
-		return left > right ? left : right ;
+template <int RTYPE, bool LHS_NA, bool RHS_NA> struct pmax_op ; 
+
+// specializations for double. 
+// we use the fact that NA < x is false
+template <>
+struct pmax_op<REALSXP,true,true>{
+	inline double operator()( double left, double right ) const {
+		return ( Rcpp::traits::is_na<REALSXP>( left ) || (left > right) ) ? left : right ;
 	}
 } ;
-template <int RTYPE, bool RHS_NA> class pmax_op<RTYPE,false,RHS_NA> {
-public:
-	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
-	
-	inline STORAGE operator()( STORAGE left, STORAGE right ) const {
-		if( Rcpp::traits::is_na<RTYPE>(right) ) return right ;
-		return left > right ? left : right ;
+template <> struct pmax_op<REALSXP,true,false> {
+	inline double operator()( double left, double right ) const {
+		return right > left ? right : left ;
 	}
 } ;
-template <int RTYPE> class pmax_op<RTYPE,false,false> {
-public:
-	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
-	
-	inline STORAGE operator()( STORAGE left, STORAGE right ) const {
+template <> struct pmax_op<REALSXP,false,true> {
+	inline double operator()( double left, double right ) const {
+		return right > left ? right : left ;
+	}
+} ;
+template <> struct pmax_op<REALSXP,false,false> {
+	inline double operator()( double left, double right ) const {
 		return left > right ? left : right ;
 	}
 } ;
 
-template <int RTYPE,bool NA> class pmax_op_Vector_Primitive {
-public:
-	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
-
-	pmax_op_Vector_Primitive( STORAGE right_ ) : 
-		right(right_) {}
-	
-	inline STORAGE operator()( STORAGE left ) const {
-		if( Rcpp::traits::is_na<RTYPE>(left) ) return left ;
-		return left > right ? left : right ;
-	}	
-		
-private:
-	STORAGE right ;
+// specializations for INTSXP. Since NA is represented as the smallest 
+// int, NA is always the smallest, so it is safe to return NA
+template <bool LHS_NA, bool RHS_NA>
+struct pmax_op<INTSXP,LHS_NA,RHS_NA> {
+    inline int operator()(int left, int right) const {
+        return left > right ? left : right ;
+    }
 } ;
 
-template <int RTYPE> class pmax_op_Vector_Primitive<RTYPE,false> {
+
+// general case
+template <int RTYPE, bool NA> class pmax_op_Vector_Primitive {
 public:
 	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
 
-	pmax_op_Vector_Primitive( STORAGE right_ ) : 
-		right(right_){}
+	pmax_op_Vector_Primitive( STORAGE right_ ) :  right(right_) {}
 	
 	inline STORAGE operator()( STORAGE left ) const {
 		return left > right ? left : right ;
@@ -93,7 +75,20 @@ public:
 private:
 	STORAGE right ;
 } ;
+// only special case we need to take care of
+template <> class pmax_op_Vector_Primitive<REALSXP,true> {
+public:
+	pmax_op_Vector_Primitive( double right_ ) :  right(right_) {}
+	
+	inline double operator()( double left ) const {
+		return ( Rcpp::traits::is_na<REALSXP>( left ) || (left > right) ) ? left : right ;
+	}	
+		
+private:
+	double right ;
+} ;
 
+    
 
 
 template <
@@ -107,13 +102,10 @@ class Pmax_Vector_Vector : public VectorBase<
 	Pmax_Vector_Vector<RTYPE,LHS_NA,LHS_T,RHS_NA,RHS_T>
 > {
 public:
-	typedef typename Rcpp::VectorBase<RTYPE,LHS_NA,LHS_T> LHS_TYPE ;
-	typedef typename Rcpp::VectorBase<RTYPE,RHS_NA,RHS_T> RHS_TYPE ;
 	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
 	typedef pmax_op<RTYPE,LHS_NA,RHS_NA> OPERATOR ;
 	
-	Pmax_Vector_Vector( const LHS_TYPE& lhs_, const RHS_TYPE& rhs_ ) : 
-		lhs(lhs_), rhs(rhs_), op() {}
+	Pmax_Vector_Vector( const LHS_T& lhs_, const RHS_T& rhs_ ) : lhs(lhs_), rhs(rhs_), op() {}
 	
 	inline STORAGE operator[]( int i ) const {
 		return op( lhs[i], rhs[i] ) ;
@@ -121,8 +113,8 @@ public:
 	inline int size() const { return lhs.size() ; }
 	         
 private:
-	const LHS_TYPE& lhs ;
-	const RHS_TYPE& rhs ;
+	const LHS_T& lhs ;
+	const RHS_T& rhs ;
 	OPERATOR op ;
 } ;
 
@@ -138,23 +130,19 @@ class Pmax_Vector_Primitive : public VectorBase<
 	Pmax_Vector_Primitive<RTYPE,LHS_NA,LHS_T>
 > {
 public:
-	typedef typename Rcpp::VectorBase<RTYPE,LHS_NA,LHS_T> LHS_TYPE ;
 	typedef typename Rcpp::traits::storage_type<RTYPE>::type STORAGE ;
 	typedef pmax_op_Vector_Primitive<RTYPE,LHS_NA> OPERATOR ;
 	
-	Pmax_Vector_Primitive( const LHS_TYPE& lhs_, STORAGE rhs_ ) : 
-		lhs(lhs_), op(rhs_), rhs(rhs_), isna( Rcpp::traits::is_na<RTYPE>(rhs_) ) {}
+	Pmax_Vector_Primitive( const LHS_T& lhs_, STORAGE rhs_ ) : lhs(lhs_), op(rhs_) {}
 	
 	inline STORAGE operator[]( int i ) const {
-		return isna ? rhs : op( lhs[i] ) ;
+		return op( lhs[i] ) ;
 	}
 	inline int size() const { return lhs.size() ; }
 	         
 private:
-	const LHS_TYPE& lhs ;
+	const LHS_T& lhs ;
 	OPERATOR op ;
-	STORAGE rhs ;
-	bool isna ;
 } ;
 
 
@@ -171,7 +159,7 @@ pmax(
 	const Rcpp::VectorBase<RTYPE,LHS_NA,LHS_T>& lhs, 
 	const Rcpp::VectorBase<RTYPE,RHS_NA,RHS_T>& rhs 
 	){
-	return sugar::Pmax_Vector_Vector<RTYPE,LHS_NA,LHS_T,RHS_NA,RHS_T>( lhs, rhs ) ;
+	return sugar::Pmax_Vector_Vector<RTYPE,LHS_NA,LHS_T,RHS_NA,RHS_T>( lhs.get_ref(), rhs.get_ref() ) ;
 }
 
 template <
@@ -183,7 +171,7 @@ pmax(
 	const Rcpp::VectorBase<RTYPE,LHS_NA,LHS_T>& lhs, 
 	typename Rcpp::traits::storage_type<RTYPE>::type rhs 
 	){
-	return sugar::Pmax_Vector_Primitive<RTYPE,LHS_NA,LHS_T>( lhs, rhs ) ;
+	return sugar::Pmax_Vector_Primitive<RTYPE,LHS_NA,LHS_T>( lhs.get_ref(), rhs ) ;
 }
 
 
@@ -196,7 +184,7 @@ pmax(
 	typename Rcpp::traits::storage_type<RTYPE>::type lhs,  
 	const Rcpp::VectorBase<RTYPE,RHS_NA,RHS_T>& rhs 
 	){
-	return sugar::Pmax_Vector_Primitive<RTYPE,RHS_NA,RHS_T>( rhs, lhs ) ;
+	return sugar::Pmax_Vector_Primitive<RTYPE,RHS_NA,RHS_T>( rhs.get_ref(), lhs ) ;
 }
 
 
